import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";
import {HomePageHeroSectionCmsRepository} from "./home-page-hero-section-cms.repository";
import {UpdateHomePageHeroSectionCmsDto} from "./dto/update-home-page-hero-section-cms.dto";
import {HomePageHeroSectionCmsEntity} from "./entities/home-page-hero-section-cms.entity";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";

@Injectable()
export class HomePageHeroSectionCmsService {
    constructor(
        @InjectRepository(HomePageHeroSectionCmsRepository)
        private readonly repository: HomePageHeroSectionCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateHomePageHeroSectionCmsDto,
        files: {
            overviewImage: UploadedMulterFileI
        }
    ): Promise<HomePageHeroSectionCmsEntity | Error> {
        if (files && files.overviewImage) {
            const overviewImage: any = await this.doSpaceService.uploadFile(files.overviewImage[0], "HomePageHeroSectionCms/");
            dto.overviewImage = overviewImage;
        }
        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<HomePageHeroSectionCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}