import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {HowItWorksSectionCmsRepository} from "./how-it-works-section-cms.repository";
import {UpdateHowItWorksSectionCmsDto} from "./dto/update-how-it-works-section-cms.dto";
import {HowItWorksSectionCmsEntity} from "./entities/how-it-works-section-cms.entity";

@Injectable()
export class HowItWorksSectionCmsService {
    constructor(
        @InjectRepository(HowItWorksSectionCmsRepository)
        private readonly repository: HowItWorksSectionCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateHowItWorksSectionCmsDto,
        files: {
            image: UploadedMulterFileI
        }
    ): Promise<HowItWorksSectionCmsEntity | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "HowItWorksSectionCms/");
            dto.image = image;
        }
        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<HowItWorksSectionCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}