import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateApplicationsIntegrationDto} from "./dto/create-applications-integration.dto";
import {UpdateApplicationsIntegrationDto} from "./dto/update-applications-integration.dto";
import {ApplicationsIntegrationEntity} from "./entity/applications-integration.entity";

@EntityRepository(ApplicationsIntegrationEntity)
export class ApplicationsIntegrationRepository extends Repository<ApplicationsIntegrationEntity> {
    async createEntity(dto: CreateApplicationsIntegrationDto): Promise<ApplicationsIntegrationEntity | Error> {
        try {
            const {
                link,
                logo,
                name
            } = dto;
            const entity = this.create({
                link,
                logo,
                name
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ApplicationsIntegrationEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateApplicationsIntegrationDto): Promise<ApplicationsIntegrationEntity | Error> {
        try {
            await this.update(id, {
                link: dto.link,
                logo: dto.logo,
                name: dto.name
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ApplicationsIntegrationEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ApplicationsIntegrationEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}