import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from '@nestjs/common';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {Constants} from 'src/utils/constants';
import {PaymentPackageService} from './payment-package.service';
import {CreatePaymentPackageDto} from './dto/create-payment-package.dto';
import {UpdatePaymentPackageDto} from './dto/update-payment-package.dto';

@Controller({path: 'payment-package', version: Constants.API_VERSION_1})
export class PaymentPackageController {
    constructor(private readonly service: PaymentPackageService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreatePaymentPackageDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdatePaymentPackageDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
