import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateScheduleIndustryDto} from "./dto/create-schedule-industry.dto";
import {UpdateScheduleIndustryDto} from "./dto/update-schedule-industry.dto";
import {ScheduleIndustryEntity} from "./entity/schedule-industry.entity";

@EntityRepository(ScheduleIndustryEntity)
export class ScheduleIndustryRepository extends Repository<ScheduleIndustryEntity> {

    async createEntity(dto: CreateScheduleIndustryDto): Promise<ScheduleIndustryEntity | Error> {
        try {
            const {
                industryName
            } = dto;
            const entity = this.create({
                industryName
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ScheduleIndustryEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ScheduleIndustryEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateScheduleIndustryDto): Promise<ScheduleIndustryEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ScheduleIndustryEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}