import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {ScheduleModelRepository} from './schedule-model.repository';
import {CreateScheduleModelDto} from './dto/create-schedule-model.dto';
import {UpdateScheduleModelDto} from './dto/update-schedule-model.dto';
import {ScheduleModelEntity} from './entity/schedule-model.entity';

@Injectable()
export class ScheduleModelService {
    constructor(
        @InjectRepository(ScheduleModelRepository)
        private readonly repository: ScheduleModelRepository
    ) {
    }

    async create(dto: CreateScheduleModelDto): Promise<ScheduleModelEntity | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
    }

    async findAll(): Promise<ScheduleModelEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async update(id: string, dto: UpdateScheduleModelDto): Promise<ScheduleModelEntity | Error> {
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", data);
    }

    async findOne(id: string): Promise<ScheduleModelEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
    }
}
