import { Body, Controller, Delete, Get, Param, Patch, Post, Req, UseGuards } from '@nestjs/common';
import { Constants } from 'src/utils/constants';
import { AuthGuard } from "../../core/guards/auth.guard";
import { CreateEventScheduleDto } from './dto/createEventSchedule.dto';
import { EventScheduleService } from './event-schedule.service';
import { UpdateEventScheduleDto } from './dto/updateEventSchedule.dto';

@Controller({ path: 'event-schedule', version: Constants.API_VERSION_1 })
export class EventScheduleController {
    constructor(private readonly service: EventScheduleService) {
    }

    @Post()
    @UseGuards(AuthGuard)
    create(@Body() dto: CreateEventScheduleDto, @Req() request: any) {
        return this.service.create(dto, request.user);
    }

    @Get()
    @UseGuards(AuthGuard)
    findAll(@Req() request: any) {
        return this.service.findAll(request.user);
    }

    @Get(':id')
    @UseGuards(AuthGuard)
    findOne(@Param('id') id: string, @Req() request: any) {
        return this.service.findOne(id, request.user);
    }

    @Patch(':id')
    @UseGuards(AuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateEventScheduleDto, @Req() request: any) {
        return this.service.update(id, dto, request.user);
    }

    @Delete(':id')
    @UseGuards(AuthGuard)
    async remove(@Param('id') id: string, @Req() request: any) {
        return await this.service.remove(id, request.user);
    }
}
