import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    Req,
    UseGuards,
    UseInterceptors
} from '@nestjs/common';
import { Constants } from 'src/utils/constants';
import { AuthGuard } from "../../core/guards/auth.guard";
import { CreateTransactionAddressDetailsDto } from './dto/create-transaction-address-details.dto';
import { TransactionAddressDetailsService } from './transaction-address-details.service';
import { UpdateTransactionAddressDetailsDto } from './dto/update-transaction-address-details.dto';

@UseGuards(AuthGuard)
@Controller({ path: 'transaction-address-details', version: Constants.API_VERSION_1 })

export default class TransactionAddressDetailsController {
    constructor(private readonly service: TransactionAddressDetailsService) {
    }

    @Post()

    create(
        @Body() dto: CreateTransactionAddressDetailsDto,
        @Req() request: any
    ) {
        return this.service.create(dto, request);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch('/:id')
    update(
        @Param('id') id: string,
        @Body() dto: UpdateTransactionAddressDetailsDto,
        @Req() request: any
    ) {
        return this.service.update(id, dto, request.user);
    }

    @Delete(':id')
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
