import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    Req,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from '@nestjs/common';
import { Constants } from 'src/utils/constants';
import { AuthGuard } from "../../core/guards/auth.guard";
import { CreateTransactionDocumentsDto } from './dto/create-transaction-documents.dto';
import { UpdateTransactionDocumentsDto } from './dto/update-transaction-documents.dto';
import { FileFieldsInterceptor } from '@nestjs/platform-express';
import { UploadedMulterFileI } from '../SpacesModule/SpacesService';
import { TransactionDocumentsService } from './transaction-documents.service';

@UseGuards(AuthGuard)
@Controller({ path: 'transaction-documents', version: Constants.API_VERSION_1 })

export default class TransactionDocumentsController {
    constructor(private readonly service: TransactionDocumentsService) {
    }

    @Post()
    @UseInterceptors(FileFieldsInterceptor([
        { name: 'file', maxCount: 1 }
    ]))
    create(
        @Body() dto: CreateTransactionDocumentsDto,
        @Req() request: any,
        @UploadedFiles() files: {
            file?: UploadedMulterFileI
        },
    ) {
        return this.service.create(dto, request.user, files);
    }

    @Get()
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @UseInterceptors(FileFieldsInterceptor([
        { name: 'file', maxCount: 1 }
    ]))
    @Patch('/:id')
    update(
        @Param('id') id: string,
        @Body() dto: UpdateTransactionDocumentsDto,
        @Req() request: any,
        @UploadedFiles() files: {
            file?: UploadedMulterFileI
        },
    ) {
        return this.service.update(id, dto, request, files);
    }

    @Delete(':id')
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
