import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { FileUploadsService } from 'src/modules/file-uploads/file-uploads.service';

import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { TransactionRepository } from '../transaction/transaction.repository';
import { TransactionDocumentsRepository } from './transaction-documents.repository';
import { CreateTransactionDocumentsDto } from './dto/create-transaction-documents.dto';
import { TransactionDocumentsEntity } from './entity/transaction-documents.entity';
import { UpdateTransactionDocumentsDto } from './dto/update-transaction-documents.dto';
import { LogsRepository } from '../transaction-logs/logs.repository';

@Injectable()
export class TransactionDocumentsService {
    constructor(
        @InjectRepository(TransactionDocumentsRepository)
        private readonly repository: TransactionDocumentsRepository,
        @InjectRepository(TransactionRepository)
        private readonly transactionRepository: TransactionRepository,
        private readonly doSpacesService: DoSpacesService,
        @InjectRepository(LogsRepository)
        private readonly logsRepository: LogsRepository
    ) {
    }

    async create(
        dto: CreateTransactionDocumentsDto,
        user: any,
        files: {
            file?: UploadedMulterFileI,
        },
    ): Promise<TransactionDocumentsEntity | Error> {
        const transaction: any = await this.transactionRepository.findByFilterQuery({ id: dto.transactionId });
        if (!transaction.length) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.file) {
            const url = await this.doSpacesService.uploadDocument(files.file[0], "TransactionNotes/");
            dto.file = url.toString();
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        // create logs
        const createLogsDto = {
            userId: user.id,
            address: transaction[0].streetAddress,
            clientName: transaction[0].clientName,
            informationChnage: "Create new document file for transaction",
            reasonForChanging: null,
        }
        await this.logsRepository.createEntity(createLogsDto);
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateTransactionDocumentsDto,
        request: any,
        files: {
            file?: UploadedMulterFileI,
        },
    ): Promise<TransactionDocumentsEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.file) {
            const url = await this.doSpacesService.uploadDocument(files.file[0], "TransactionNotes/");
            dto.file = url.toString();
        }
        const { transactionId, ...updateDto } = dto;
        const data = await this.repository.updateEntity(id, updateDto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<TransactionDocumentsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<TransactionDocumentsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
