import { EntityRepository, getConnection, Repository } from "typeorm";
import { LogsEntity } from "./entity/logs.entity";
import { CreateLogDto } from "./dto/create-log.dto";

@EntityRepository(LogsEntity)
export class LogsRepository extends Repository<LogsEntity> {

    async createEntity(dto: CreateLogDto): Promise<LogsEntity | Error> {
        try {
            const {
                userId,
                address,
                clientName,
                informationChnage,
                reasonForChanging,
            } = dto;
            const entity = this.create({
                userId,
                address,
                clientName,
                informationChnage,
                reasonForChanging,
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any, offset: number, limit: number): Promise<LogsEntity[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false },
                order: { createdAt: -1 },
                skip: offset,
                take: limit,
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(LogsEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}