import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { TransactionRepository } from '../transaction/transaction.repository';
import { CreateTransactionTimelineDetailsDto } from './dto/create-transaction-timeline-details.dto';
import { TransactionTimelineDetailsEntity } from './entity/transaction-timeline-details.entity';
import { TransactionTimelineDetailsRepository } from './transaction-timeline-details.repository';
import { UpdateTransactionTimelineDetailsDto } from './dto/update-transaction-timeline-details.dto';
import { LogsRepository } from '../transaction-logs/logs.repository';
import { TransactionPaymentHistoryRepository } from '../transaction-payment-history/transaction-payment-history.repository';
import { paymentType } from 'src/enum-type/payment-type.enum';
import { UsersRepository } from '../users/user.repository';
import { formatDateYearMonthDay } from "src/helper/common-helper";

@Injectable()
export class TransactionTimelineDetailsService {
    constructor(
        @InjectRepository(TransactionTimelineDetailsRepository)
        private readonly repository: TransactionTimelineDetailsRepository,
        @InjectRepository(TransactionRepository)
        private readonly transactionRepository: TransactionRepository,
        @InjectRepository(LogsRepository)
        private readonly logsRepository: LogsRepository,
        @InjectRepository(TransactionPaymentHistoryRepository)
        private readonly transactionPaymentHistoryRepository: TransactionPaymentHistoryRepository,
        @InjectRepository(UsersRepository)
        private readonly usersRepository: UsersRepository
    ) {
    }

    async create(
        dto: CreateTransactionTimelineDetailsDto,
        request: any
    ): Promise<TransactionTimelineDetailsEntity | Error> {
        const transaction: any = await this.transactionRepository.findByFilterQuery({ id: dto.transactionId });
        if (!transaction.length) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateTransactionTimelineDetailsDto,
        user: any
    ): Promise<TransactionTimelineDetailsEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const { transactionId, ...updateDto } = dto;
        const data = await this.repository.updateEntity(id, updateDto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        // find specifice transaction
        const tran = await this.transactionRepository.findOneByFilterQuery({ id: res.transactionId });

        // update transaction price
        try {
            if(dto?.price && tran && tran?.[0]?.id) {
                await this.transactionRepository.update(res.transactionId, { price: +dto?.price });
            }
        } catch (error) {
            console.log(error);
        }

        // create logs
        let createLog = {
            userId: tran[0].userId,
            address: tran[0].streetAddress,
            clientName: tran[0].clientName,
            informationChnage: null,
            reasonForChanging: null,
        }
        if(dto?.closingDate) {
            const createLogsDto = {
                ...createLog,
                informationChnage: `Updated transaction timeline closing date to ${formatDateYearMonthDay(dto?.closingDate)}`,
            }
            createLogsDto.informationChnage = `Updated transaction timeline closing date to ${formatDateYearMonthDay(dto?.closingDate)}`;
            await this.logsRepository.createEntity(createLogsDto);
        }
        if(dto?.inspectionDate) {
            const createLogsDto = {
                ...createLog,
                informationChnage: `Updated transaction timeline inspection date to ${formatDateYearMonthDay(dto?.inspectionDate)}`
            }
            await this.logsRepository.createEntity(createLogsDto);
        }
        if(dto?.downPayment) {
            const createLogsDto = {
                ...createLog,
                informationChnage: `Updated transaction timeline down payment date to ${formatDateYearMonthDay(dto?.downPayment)}`
            }
            await this.logsRepository.createEntity(createLogsDto);
        }
        if(dto?.deposit) {
            const createLogsDto = {
                ...createLog,
                informationChnage: `Updated transaction timeline deposit to $${dto?.deposit}`
            }
            await this.logsRepository.createEntity(createLogsDto);
        }
        if(dto?.sellerConcessions) {
            const createLogsDto = {
                ...createLog,
                informationChnage: `Updated transaction timeline Seller concessions to $${dto?.sellerConcessions}`
            }
            await this.logsRepository.createEntity(createLogsDto);
        }
        if(dto?.closing) {
            const createLogsDto = {
                ...createLog,
                informationChnage: `Updated transaction timeline Down Payment to $${dto?.closing}`
            }
            await this.logsRepository.createEntity(createLogsDto);
        }
        if(dto?.price) {
            const createLogsDto = {
                ...createLog,
                informationChnage: `Updated transaction timeline price to $${dto?.price}`
            }
            await this.logsRepository.createEntity(createLogsDto);
        }
        if(dto?.methodOfPay) {
            const createLogsDto = {
                ...createLog,
                informationChnage: `Updated transaction timeline method of pay to ${dto?.methodOfPay}`
            }
            await this.logsRepository.createEntity(createLogsDto);
        }
        // if payment then, generate payment history
        const paymentHistory = await this.transactionPaymentHistoryRepository.findLatestEntity(res.transactionId);
        if(!paymentHistory) {
            if(dto.deposit) {
                const paymentHistoryDto = {
                    paymentType: paymentType.deposit,
                    transactionId: res.transactionId,
                    userId: tran[0].userId,
                    transactionTimelineId: id,
                    amount: dto.deposit
                }
                await this.transactionPaymentHistoryRepository.createEntity(paymentHistoryDto);
            } 
            if(dto.sellerConcessions) {
                const paymentHistoryDto = {
                    paymentType: paymentType.sellerConcessions,
                    transactionId: res.transactionId,
                    userId: tran[0].userId,
                    transactionTimelineId: id,
                    amount: dto.sellerConcessions
                }
                await this.transactionPaymentHistoryRepository.createEntity(paymentHistoryDto);
            }
            if(dto.closing) {
                const paymentHistoryDto = {
                    paymentType: paymentType.closing,
                    transactionId: res.transactionId,
                    userId: tran[0].userId,
                    transactionTimelineId: id,
                    amount: dto.closing
                }
                await this.transactionPaymentHistoryRepository.createEntity(paymentHistoryDto);
            } 
        } else {
            if(dto.deposit && dto.deposit !== res.deposit) {
                const paymentHistoryDto = {
                    paymentType: paymentType.deposit,
                    transactionId: res.transactionId,
                    userId: tran[0].userId,
                    transactionTimelineId: id,
                    amount: dto.deposit
                }
                await this.transactionPaymentHistoryRepository.createEntity(paymentHistoryDto);
            } 
            if(dto.sellerConcessions && dto.sellerConcessions !== res.sellerConcessions) {
                const paymentHistoryDto = {
                    paymentType: paymentType.sellerConcessions,
                    transactionId: res.transactionId,
                    userId: tran[0].userId,
                    transactionTimelineId: id,
                    amount: dto.sellerConcessions
                }
                await this.transactionPaymentHistoryRepository.createEntity(paymentHistoryDto);
            }
            if(dto.closing && dto.closing !== res.closing) {
                const paymentHistoryDto = {
                    paymentType: paymentType.closing,
                    transactionId: res.transactionId,
                    userId: tran[0].userId,
                    transactionTimelineId: id,
                    amount: dto.closing
                }
                await this.transactionPaymentHistoryRepository.createEntity(paymentHistoryDto);
            } 
        }

        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<TransactionTimelineDetailsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<TransactionTimelineDetailsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
